// Copyright (C) 2021  Nexedi SA and Contributors.
//                     Kirill Smelkov <kirr@nexedi.com>
//
// This program is free software: you can Use, Study, Modify and Redistribute
// it under the terms of the GNU General Public License version 3, or (at your
// option) any later version, as published by the Free Software Foundation.
//
// You can also Link and Combine this program with other software covered by
// the terms of any of the Free Software licenses or any of the Open Source
// Initiative approved licenses and Convey the resulting work. Corresponding
// source of such a combination shall include the source code for all other
// software used.
//
// This program is distributed WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// See COPYING file for full licensing terms.
// See https://www.nexedi.com/licensing for rationale and options.

package blib
// range of keys.

import (
	"fmt"
)


// KeyRange represents [lo,hi) Key range.
type KeyRange struct {
	Lo  Key
	Hi_ Key // NOTE _not_ hi) to avoid overflow at ∞;  hi = hi_ + 1
}


// Has returns whether key k belongs to the range.
func (r *KeyRange) Has(k Key) bool {
	return (r.Lo <= k && k <= r.Hi_)
}

// Empty returns whether key range is empty.
func (r *KeyRange) Empty() bool {
	hi := r.Hi_
	if hi == KeyMax {
		// [x,∞] cannot be empty because max x is ∞ and [∞,∞] has one element: ∞
		return false
	}
	hi++ // no overflow
	return r.Lo >= hi
}


func (r KeyRange) String() string {
	var shi string
	if r.Hi_ == KeyMax {
		shi = KStr(r.Hi_) // ∞
	} else {
		shi = fmt.Sprintf("%d", r.Hi_+1)
	}
	return fmt.Sprintf("[%s,%s)", KStr(r.Lo), shi)
}
